//--------------------------------------------------------------
//  IIR フィルタを利用する二相発振器のクラス
//
//  2019/04/11, Copyright (c) 2019 MIKAMI, Naoki
//--------------------------------------------------------------

#include "mbed.h"

#ifndef QUAD_PHASE_OSC_IIR_HPP
#define QUAD_PHASE_OSC_IIR_HPP

namespace Mikami
{
    class QuadOscIir
    {
    public:
        // コンストラクタ
        //      f0：     周波数，単位：Hz
        //      ts：     標本化間隔，単位：μs
        //      amp：    振幅
        QuadOscIir(float f0, float ts, float amp = 1)
            : TS_(ts*1.0E-6f), A0_(amp) { Set(f0); }

        // sin/cos 発生，実行時間の実測値：91 ns
        void Generate(float &sinX, float &cosX)
        {
            RecursivePart();
            sinX = b1S_*un2_;
            cosX = A0_*un1_ + b1C_*un2_;
        }
    
        // sin 発生，実行時間の実測値：69 ns
        float GenerateSin()
        {
            RecursivePart();
            return b1S_*un2_;
        }

        // cis 発生，実行時間の実測値：80 ns
        float GenerateCos()
        {
            RecursivePart();
            return A0_*un1_ + b1C_*un2_;
        }

        // 出力周波数の設定，周波数を変更し再スタートする場合も使える
        //      f0：周波数，単位：Hz
        void Set(float f0)
        {
            static const float pi2 = 6.283185f;
            a1_ = 2.0f*cosf(pi2*f0*TS_);
            b1S_ = A0_*sinf(pi2*f0*TS_);
            b1C_ = -A0_*cosf(pi2*f0*TS_);
            un1_ = 1.0f;
            un2_ = 0.0f;
        }

    private:
        const float TS_;        // 標本化間隔
        const float A0_;        // 振幅
        float a1_, b1S_, b1C_;
        float un1_, un2_;

        // u[n] = a1*u[n-1] - u[n-2] の計算
        void RecursivePart()
        {
            float un = a1_*un1_ - un2_;
            un2_ = un1_;    // u[n-2] ← u[n-1]
            un1_ = un;      // u[n-1] ← u[n]
        }
    };
}
#endif  // QUAD_PHASE_OSC_IIR_HPP
